@@ -9,7 +9,9 @@ from .guide import GUIDE
 from .mlpae import MLPAE
 from .ocgnn import OCGNN
 from .one import ONE
-
+from .conad import CONAD
+from .cola import CoLA
+from .anemone import ANEMONE
 
 
 
